from strawberry.dataloader import DataLoader

from bookshelves.models import Book


async def load_books(cat_ids: list[int]) -> list[list[Book]]:
    """Bookのデータローダに渡すバッチ関数"""
    # SQLを発行して、
    # CategoryとBookを結合した結果をIN句で一括取得する
    books_q = Book.objects.filter(category_id__in=cat_ids).select_related("category")
    books = [book async for book in books_q]

    # カテゴリIDがキーで対応する本(複数)が
    # 値の辞書を作成する
    cat_id_to_books_dict: dict[int, list[Book]] = {cat_id: [] for cat_id in cat_ids}
    for book in books:
        cat_id_to_books_dict[book.category.id].append(book)
    # 作成した辞書を使って
    # 引数のID順に揃えたリストを作成して返す
    return [cat_id_to_books_dict[cat_id] for cat_id in cat_ids]


# Bookのデータローダ
books_loader = DataLoader(load_fn=load_books)
