import strawberry
import strawberry_django
from django.contrib.auth.models import User
from graphql import GraphQLError

from bookshelves.models import Category
from bookshelves.types import AddBookInput, AddCategoryInput, BookType, CategoryType, UserType


@strawberry.type
class CategoryNameValidationError:
    category_name: str
    message: str


@strawberry.type
class Query:
    @strawberry_django.field
    def me(self, info) -> UserType | None:
        user = info.context.request.user
        if user.is_authenticated:
            return User.objects.get(pk=user.pk)
        else:
            return None


@strawberry.type
class Mutation:
    @strawberry.mutation
    def add_category(
        self, info, input: AddCategoryInput
    ) -> CategoryType | CategoryNameValidationError:
        user = info.context.request.user
        if len(input.name) > 16:
            return CategoryNameValidationError(
                category_name=input.name, message="カテゴリ名は16文字以下である必要があります。"
            )
        else:
            return Category.objects.acreate(user=user, name=input.name)

    @strawberry.mutation
    def add_book(self, info, input: AddBookInput) -> BookType:
        # raise Exception("未実装です。")
        raise GraphQLError("未実装です。", extensions={"code": "UNIMPLEMENTED"})


schema = strawberry.Schema(query=Query, mutation=Mutation)
