import strawberry
import strawberry_django
from django.contrib.auth.models import User

from bookshelves import services
from bookshelves.models import Book, Category


@strawberry_django.input(Book)
class AddBookInput:
    category_id: int
    name: strawberry.auto


@strawberry_django.type(Book)
class BookType:
    id: strawberry.auto
    name: strawberry.auto


@strawberry_django.input(Category)
class AddCategoryInput:
    name: strawberry.auto


@strawberry_django.type(Category)
class CategoryType:
    id: strawberry.auto
    name: strawberry.auto

    @strawberry_django.field
    async def books(self) -> list[BookType]:
        return await services.find_my_books_of_categories(self.id)


@strawberry_django.type(User)
class UserType:
    id: strawberry.auto
    email: strawberry.auto
    username: strawberry.auto

    @strawberry_django.field
    async def categories(self) -> list[CategoryType]:
        return await services.find_my_categories(self.id)
