from pathlib import Path
from unittest import mock

import numpy as np
import pytest


@pytest.fixture(autouse=True)
def setup_random_seed():
    """乱数によって結果が変わらないようにシードを固定"""
    np.random.seed(1)


@pytest.fixture()
def target():
    from samplecommand import process

    return process


@pytest.mark.parametrize(
    "case_name",
    # 入力ケースを増やす場合は以下を追加
    ["case01"],
)
def test_snapshot(target, tmp_path, snapshot, case_name):
    """入力値に対する出力値をスナップショットテストする"""
    # arrange
    args = mock.Mock()
    args.in_dir = Path("tests/test_snapshot/inputs") / case_name
    args.out_dir = tmp_path

    # act
    target(args)

    # assert
    # 一時ディレクトリから実行結果を読み込む
    result_txt = (tmp_path / "result.csv").read_text()
    # snapshotモジュールで結果を比較
    snapshot.assert_match(result_txt, f"{case_name} result")
